@@ -108,11 +108,10 @@ char *esc_key(void)
   return buf;
 }
 
-/*ARGSUSED*/
 static void get_alrm(int dummy)
 {
   (void)dummy;
-  errno = ETIME;
+  errno = ETIMEDOUT;
   longjmp(albuf, 1);
 }
 
@@ -416,7 +415,7 @@ void init_emul(int type, int do_init)
   /* Install and reset the terminal emulator. */
   if (do_init) {
     vt_install(do_output, kb_handler, us);
-    vt_init(type, tfcolor, tbcolor, us->wrap, addlf);
+    vt_init(type, tfcolor, tbcolor, us->wrap, addlf, addcr);
   } else
     vt_pinit(us, -1, -1);
 
@@ -689,7 +688,6 @@ int do_terminal(void)
 {
   char buf[128];
   int buf_offset = 0;
-  char *ptr;
   int c;
   int x;
   int blen;
@@ -770,9 +768,10 @@ dirty_goto:
 
     /* Data from the modem to the screen. */
     if ((x & 1) == 1) {
+      char obuf[sizeof(buf)];
+      char *ptr;
 
       if (using_iconv()) {
-        char obuf[sizeof(buf)];
         char *otmp = obuf;
         size_t output_len = sizeof(obuf);
         size_t input_len = blen;
@@ -780,13 +779,20 @@ dirty_goto:
         ptr = buf;
         do_iconv(&ptr, &input_len, &otmp, &output_len);
 
-        if (input_len) { // something remained, we need to adapt buf accordingly
-          memmove(buf, ptr, input_len);
-          buf_offset = input_len;
-        }
+        // something happened at all?
+        if (output_len < sizeof(obuf))
+          {
+            if (input_len)
+              { // something remained, we need to adapt buf accordingly
+                memmove(buf, ptr, input_len);
+                buf_offset = input_len;
+              }
 
-        blen = sizeof(obuf) - output_len;
-        ptr = obuf;
+            blen = sizeof(obuf) - output_len;
+            ptr = obuf;
+          }
+	else
+	  ptr = buf;
       } else {
         ptr = buf;
       }
@@ -801,7 +807,15 @@ dirty_goto:
         }
         if (P_PARITY[0] == 'M' || P_PARITY[0] == 'S')
           *ptr &= 0x7f;
-        vt_out(*ptr++);
+        if (display_hex) {
+          unsigned char c = *ptr++;
+          unsigned char u = c >> 4;
+          c &= 0xf;
+          vt_out(u > 9 ? 'a' + (u - 10) : '0' + u);
+          vt_out(c > 9 ? 'a' + (c - 10) : '0' + c);
+          vt_out(' ');
+        } else
+          vt_out(*ptr++);
         if (zauto && zsig[zpos] == 0) {
           dirflush = 1;
           keyboard(KSTOP, 0);
