@@ -817,6 +817,7 @@ static void helpthem(void)
     "  -l, --ansi             : literal; assume screen uses non IBM-PC character set\n"
     "  -L, --iso              : don't assume screen uses ISO8859\n"
     "  -w, --wrap             : Linewrap on\n"
+    "  -H, --displayhex       : display output in hex\n"
     "  -z, --statline         : try to use terminal's status line\n"
     "  -7, --7bit             : force 7bit mode\n"
     "  -8, --8bit             : force 8bit mode\n"
@@ -828,6 +829,7 @@ static void helpthem(void)
     "  -p, --ptty=TTYP        : connect to pseudo terminal\n"
     "  -C, --capturefile=FILE : start capturing to FILE\n"
     "  -T, --disabletime      : disable display of online time\n"
+    "  -R, --remotecharset    : character set of communication partner\n"
     "  -v, --version          : output version information and exit\n"
     "  configuration          : configuration file to use\n\n"
     "These options can also be specified in the MINICOM environment variable.\n"),
@@ -845,7 +847,7 @@ static void helpthem(void)
 
 static void set_addlf(int val)
 {
-  vt_set(val, -1, -1, -1, -1, -1, -1, -1);
+  vt_set(val, -1, -1, -1, -1, -1, -1, -1, -1);
 }
 
 /* Toggle linefeed addition.  Can be called through the menu, or by a macro. */
@@ -855,9 +857,21 @@ void toggle_addlf(void)
   set_addlf(addlf);
 }
 
+static void set_addcr(int val)
+{
+  vt_set(-1, -1, -1, -1, -1, -1, -1, -1, val);
+}
+
+/* Toggle carriagereturn addition.  Can be called through the menu, or by a macro. */
+void toggle_addcr(void)
+{
+  addcr = !addcr;
+  set_addcr(addcr);
+}
+
 static void set_local_echo(int val)
 {
-  vt_set(-1, -1, -1, -1, val, -1 ,-1, -1);
+  vt_set(-1, -1, -1, -1, val, -1 ,-1, -1, -1);
 }
 
 /* Toggle local echo.  Can be called through the menu, or by a macro. */
@@ -869,7 +883,7 @@ void toggle_local_echo(void)
 
 static void set_line_timestamp(int val)
 {
-  vt_set(-1, -1, -1, -1 ,-1, -1, -1, val);
+  vt_set(-1, -1, -1, -1 ,-1, -1, -1, val, -1);
 }
 
 /* Toggle host timestamping on/off */
@@ -882,6 +896,19 @@ static void toggle_line_timestamp(void)
 
 /* -------------------------------------------- */
 
+static void do_iconv_just_copy(char **inbuf, size_t *inbytesleft,
+                               char **outbuf, size_t *outbytesleft)
+{
+  while (*outbytesleft && *inbytesleft)
+    {
+      *(*outbuf) = *(*inbuf);
+      ++(*outbuf);
+      ++(*inbuf);
+      --(*outbytesleft);
+      --(*inbytesleft);
+    }
+}
+
 #ifdef HAVE_ICONV
 static iconv_t iconv_rem2local;
 static int iconv_enabled;
@@ -916,8 +943,11 @@ static void init_iconv(const char *remote_charset)
            "%s//TRANSLIT", tmp);
   local_charset[sizeof(local_charset) - 1] = 0;
 
-  //printf("Remote charset: %s\n", remote_charset);
-  //printf("Local charset: %s\n", local_charset);
+  if (0)
+    {
+      printf("Remote charset: %s\n", remote_charset);
+      printf("Local charset: %s\n", local_charset);
+    }
 
   iconv_rem2local = iconv_open(local_charset, remote_charset);
   if (iconv_rem2local != (iconv_t)-1)
@@ -933,16 +963,15 @@ void do_iconv(char **inbuf, size_t *inbytesleft,
   if (!iconv_enabled
       || iconv(iconv_rem2local,
                inbuf, inbytesleft,
-               outbuf, outbytesleft) == (size_t)-1) {
-    memcpy(*outbuf, *inbuf,
-           *outbytesleft < *inbytesleft ? *outbytesleft : *inbytesleft);
+               outbuf, outbytesleft) == (size_t)-1)
+    {
+      do_iconv_just_copy(inbuf, inbytesleft, outbuf, outbytesleft);
 
-    // in case of error re-init conversion descriptor
-    // (will this work?)
-    if (iconv_enabled)
-      iconv(iconv_rem2local, NULL, NULL, NULL, NULL);
-    return;
-  }
+      // in case of error re-init conversion descriptor
+      // (will this work?)
+      if (iconv_enabled)
+	iconv(iconv_rem2local, NULL, NULL, NULL, NULL);
+    }
 }
 
 static void close_iconv(void)
@@ -965,7 +994,7 @@ static void init_iconv(const char *remote_charset)
 void do_iconv(char **inbuf, size_t *inbytesleft,
               char **outbuf, size_t *outbytesleft)
 {
-  (void)inbuf; (void)outbuf; (void)inbytesleft; (void)outbytesleft;
+  do_iconv_just_copy(inbuf, inbytesleft, outbuf, outbytesleft);
 }
 
 static void close_iconv(void)
@@ -1017,6 +1046,7 @@ int main(int argc, char **argv)
     { "8bit",          no_argument,       NULL, '8' },
     { "version",       no_argument,       NULL, 'v' },
     { "wrap",          no_argument,       NULL, 'w' },
+    { "displayhex",    no_argument,       NULL, 'H' },
     { "disabletime",   no_argument,       NULL, 'T' },
     { "baudrate",      required_argument, NULL, 'b' },
     { "device",        required_argument, NULL, 'D' },
@@ -1038,8 +1068,10 @@ int main(int argc, char **argv)
   stdattr = XA_NORMAL;
   us = NULL;
   addlf = 0;
+  addcr = 0;
   line_timestamp = 0;
   wrapln = 0;
+  display_hex = 0;
   disable_online_time = 0;
   local_echo = 0;
   strcpy(capname, "minicom.cap");
@@ -1120,7 +1152,7 @@ int main(int argc, char **argv)
 
   do {
     /* Process options with getopt */
-    while ((c = getopt_long(argk, args, "v78zhlLsomMb:wTc:a:t:d:p:C:S:D:R:",
+    while ((c = getopt_long(argk, args, "v78zhlLsomMHb:wTc:a:t:d:p:C:S:D:R:",
                             long_options, NULL)) != EOF)
       switch(c) {
 	case 'v':
@@ -1220,7 +1252,7 @@ int main(int argc, char **argv)
             exit(1);
           }
           docap = 1;
-          vt_set(addlf, -1, docap, -1, -1, -1, -1, -1);
+          vt_set(addlf, -1, docap, -1, -1, -1, -1, -1, addcr);
           break;
         case 'S': /* start Script */
           strncpy(scr_name, optarg, 33);
@@ -1234,6 +1266,9 @@ int main(int argc, char **argv)
         case 'w': /* Linewrap on */
           wrapln = 1;
           break;
+        case 'H': /* Display in hex */
+          display_hex = 1;
+          break;
         case 'T': /* disable online time */
           disable_online_time = 1;
           break;
@@ -1262,7 +1297,7 @@ int main(int argc, char **argv)
 
   if (screen_iso && screen_ibmpc)
     /* init VT */
-    vt_set(-1, -1, -1, -1, -1, -1, 1, -1);
+    vt_set(-1, -1, -1, -1, -1, -1, 1, -1, -1);
 
   /* Avoid fraude ! */	
   for (s = use_port; *s; s++)
@@ -1293,6 +1328,15 @@ int main(int argc, char **argv)
   /* Set default terminal behaviour */
   addlf      = strcasecmp(P_ADDLINEFEED, "yes") == 0;
   local_echo = strcasecmp(P_LOCALECHO,   "yes") == 0;
+  addcr      = strcasecmp(P_ADDCARRIAGERETURN, "yes") == 0;
+
+  /* -w overrides config file */
+  if (!wrapln)
+    wrapln = strcasecmp(P_LINEWRAP, "yes") == 0;
+
+  /* -H overrides config file */
+  if (!display_hex)
+    display_hex = strcasecmp(P_DISPLAYHEX, "yes") == 0;
 
   /* After reading in the config via read_parms we can possibly overwrite
    * the baudrate with a value given at the cmdline */
@@ -1304,7 +1348,7 @@ int main(int argc, char **argv)
     }
   }
 
-  /* New we can also overwrite the device name, if one was given */
+  /* Now we can also overwrite the device name, if one was given */
   if (cmdline_device) {
     strncpy(P_PORT, cmdline_device, sizeof(P_PORT));
     P_PORT[sizeof(P_PORT) - 1] = 0;
@@ -1412,7 +1456,22 @@ int main(int argc, char **argv)
 #if defined (__DATE__) && defined (__TIME__)
   mc_wprintf(us, "%s %s, %s.\r\n",_("Compiled on"), __DATE__,__TIME__);
 #endif
-  mc_wprintf(us, "%s %s\r\n", _("Port"), P_PORT);
+  {
+    struct stat st;
+    char port_date[20] = "";
+    if (stat(P_PORT, &st) == 0)
+      {
+	time_t t = time(NULL);
+        struct tm tm;
+        if (   st.st_mtime + 20 * 60 * 60 > t
+            && localtime_r(&st.st_mtime, &tm))
+            {
+              strftime(port_date, sizeof(port_date), ", %T", &tm);
+              port_date[sizeof(port_date) - 1] = 0;
+            }
+      }
+    mc_wprintf(us, "%s %s%s\r\n", _("Port"), P_PORT, port_date);
+  }
   if (using_iconv())
     mc_wprintf(us, "%s\r\n", _("Using character set conversion"));
   mc_wprintf(us, _("\nPress %sZ for help on special keys%c\n\n"),esc_key(),'\r');
@@ -1439,6 +1498,11 @@ dirty_goto:
         s = addlf ?  _("Add linefeed ON") : _("Add linefeed OFF");
         status_set_display(s, 0);
         break;
+      case 'u': /* Add carriage return */
+        toggle_addcr();
+        s = addcr ?  _("Add carriage return ON") : _("Add carriage return OFF");
+        status_set_display(s, 0);
+        break;
       case 'e': /* Local echo on/off. */
         toggle_local_echo();
         s = local_echo ?  _("Local echo ON") : _("Local echo OFF");
@@ -1527,7 +1591,7 @@ dirty_goto:
           if (c == 1)
             docap = 0;
         }
-        vt_set(addlf, -1, docap, -1, -1, -1, -1, -1);
+        vt_set(addlf, -1, docap, -1, -1, -1, -1, -1, addcr);
         break;
       case 'p': /* Set parameters */
         get_bbp(P_BAUDRATE, P_BITS, P_PARITY, P_STOPB, 0);
@@ -1552,7 +1616,7 @@ dirty_goto:
         break;
       case 'w': /* Line wrap on-off */
         c = !us->wrap;
-        vt_set(addlf, c, docap, -1, -1, -1, -1, -1);
+        vt_set(addlf, c, docap, -1, -1, -1, -1, -1, addcr);
         s = c ? _("Linewrap ON") : _("Linewrap OFF");
 	status_set_display(s, 0);
         break;
